@@ -4,10 +4,10 @@ extern crate num;
 use array::Array;
 use dim4::Dim4;
 use defines::AfError;
-use defines::Aftype;
+use error::HANDLE_ERROR;
 use self::libc::{uint8_t, c_int, c_uint, c_double};
 use self::num::Complex;
-
+use util::HasAfEnum;
 use std::vec::Vec;
 
 type MutAfArray = *mut self::libc::c_longlong;
@@ -69,87 +69,77 @@ extern {
 }
 
 pub trait ConstGenerator {
-    fn generate(&self, dims: Dim4) -> Result<Array, AfError>;
+    fn generate(&self, dims: Dim4) -> Array;
 }
 
 #[allow(unused_mut)]
 impl ConstGenerator for i64 {
-    fn generate(&self, dims: Dim4) -> Result<Array, AfError> {
+    fn generate(&self, dims: Dim4) -> Array {
         unsafe {
             let mut temp: i64 = 0;
             let err_val = af_constant_long(&mut temp as MutAfArray, *self as Intl,
                                            dims.ndims() as c_uint,
                                            dims.get().as_ptr() as *const DimT);
-            match err_val {
-                0 => Ok(Array::from(temp)),
-                _ => Err(AfError::from(err_val)),
-            }
+            HANDLE_ERROR(AfError::from(err_val));
+            Array::from(temp)
         }
     }
 }
 
 #[allow(unused_mut)]
 impl ConstGenerator for u64 {
-    fn generate(&self, dims: Dim4) -> Result<Array, AfError> {
+    fn generate(&self, dims: Dim4) -> Array {
         unsafe {
             let mut temp: i64 = 0;
             let err_val = af_constant_ulong(&mut temp as MutAfArray, *self as Uintl,
                                             dims.ndims() as c_uint,
                                             dims.get().as_ptr() as *const DimT);
-            match err_val {
-                0 => Ok(Array::from(temp)),
-                _ => Err(AfError::from(err_val)),
-            }
+            HANDLE_ERROR(AfError::from(err_val));
+            Array::from(temp)
         }
     }
 }
 
 #[allow(unused_mut)]
 impl ConstGenerator for Complex<f32> {
-    fn generate(&self, dims: Dim4) -> Result<Array, AfError> {
+    fn generate(&self, dims: Dim4) -> Array {
         unsafe {
             let mut temp: i64 = 0;
             let err_val = af_constant_complex(&mut temp as MutAfArray,
                                               (*self).re as c_double, (*self).im as c_double,
                                               dims.ndims() as c_uint,
                                               dims.get().as_ptr() as *const DimT, 1);
-            match err_val {
-                0 => Ok(Array::from(temp)),
-                _ => Err(AfError::from(err_val)),
-            }
+            HANDLE_ERROR(AfError::from(err_val));
+            Array::from(temp)
         }
     }
 }
 
 #[allow(unused_mut)]
 impl ConstGenerator for Complex<f64> {
-    fn generate(&self, dims: Dim4) -> Result<Array, AfError> {
+    fn generate(&self, dims: Dim4) -> Array {
         unsafe {
             let mut temp: i64 = 0;
             let err_val = af_constant_complex(&mut temp as MutAfArray,
                                               (*self).re as c_double, (*self).im as c_double,
                                               dims.ndims() as c_uint,
                                               dims.get().as_ptr() as *const DimT, 3);
-            match err_val {
-                0 => Ok(Array::from(temp)),
-                _ => Err(AfError::from(err_val)),
-            }
+            HANDLE_ERROR(AfError::from(err_val));
+            Array::from(temp)
         }
     }
 }
 
 #[allow(unused_mut)]
 impl ConstGenerator for bool {
-    fn generate(&self, dims: Dim4) -> Result<Array, AfError> {
+    fn generate(&self, dims: Dim4) -> Array {
         unsafe {
             let mut temp: i64 = 0;
             let err_val = af_constant(&mut temp as MutAfArray, *self as c_int as c_double,
                                       dims.ndims() as c_uint,
                                       dims.get().as_ptr() as *const DimT, 4);
-            match err_val {
-                0 => Ok(Array::from(temp)),
-                _ => Err(AfError::from(err_val)),
-            }
+            HANDLE_ERROR(AfError::from(err_val));
+            Array::from(temp)
         }
     }
 }
@@ -158,16 +148,14 @@ macro_rules! cnst {
     ($rust_type:ty, $ffi_type:expr) => (
         #[allow(unused_mut)]
         impl ConstGenerator for $rust_type {
-            fn generate(&self, dims: Dim4) -> Result<Array, AfError> {
+            fn generate(&self, dims: Dim4) -> Array {
                 unsafe {
                     let mut temp: i64 = 0;
                     let err_val = af_constant(&mut temp as MutAfArray, *self as c_double,
                                               dims.ndims() as c_uint,
                                               dims.get().as_ptr() as *const DimT, $ffi_type);
-                    match err_val {
-                        0 => Ok(Array::from(temp)),
-                        _ => Err(AfError::from(err_val)),
-                    }
+                    HANDLE_ERROR(AfError::from(err_val));
+                    Array::from(temp)
                 }
             }
         }
@@ -207,7 +195,7 @@ cnst!(u16 , 11);
 /// # Return Values
 ///
 /// An Array of given dimensions with constant value
-pub fn constant<T : ConstGenerator>(cnst: T, dims: Dim4) -> Result<Array, AfError> {
+pub fn constant<T : ConstGenerator>(cnst: T, dims: Dim4) -> Array {
     cnst.generate(dims)
 }
 
@@ -220,21 +208,19 @@ pub fn constant<T : ConstGenerator>(cnst: T, dims: Dim4) -> Result<Array, AfErro
 /// - `dims` is the size of Array
 /// - `seq_dim` is the dimension along which range values are populated, all values along other
 /// dimensions are just repeated
-/// - `aftype` is the type of the Array.
 ///
 /// # Return Values
 /// Array
 #[allow(unused_mut)]
-pub fn range(dims: Dim4, seq_dim: i32, aftype: Aftype) -> Result<Array, AfError> {
+pub fn range<T: HasAfEnum>(dims: Dim4, seq_dim: i32) -> Array {
     unsafe {
+        let aftype = T::get_af_dtype();
         let mut temp: i64 = 0;
-        let err_val =af_range(&mut temp as MutAfArray,
+        let err_val = af_range(&mut temp as MutAfArray,
                               dims.ndims() as c_uint, dims.get().as_ptr() as *const DimT,
                               seq_dim as c_int, aftype as uint8_t);
-        match err_val {
-            0 => Ok(Array::from(temp)),
-            _ => Err(AfError::from(err_val)),
-        }
+        HANDLE_ERROR(AfError::from(err_val));
+        Array::from(temp)
     }
 }
 
@@ -246,63 +232,55 @@ pub fn range(dims: Dim4, seq_dim: i32, aftype: Aftype) -> Result<Array, AfError>
 ///
 /// - `dims` is the dimensions of the sequence to be generated
 /// - `tdims` is the number of repitions of the unit dimensions
-/// - `aftype` is the type of Array to generate
 ///
 /// # Return Values
 ///
 /// Array
 #[allow(unused_mut)]
-pub fn iota(dims: Dim4, tdims: Dim4, aftype: Aftype) -> Result<Array, AfError> {
+pub fn iota<T: HasAfEnum>(dims: Dim4, tdims: Dim4) -> Array {
     unsafe {
+        let aftype = T::get_af_dtype();
         let mut temp: i64 = 0;
         let err_val =af_iota(&mut temp as MutAfArray,
                              dims.ndims() as c_uint, dims.get().as_ptr() as *const DimT,
                              tdims.ndims() as c_uint, tdims.get().as_ptr() as *const DimT,
                              aftype as uint8_t);
-        match err_val {
-            0 => Ok(Array::from(temp)),
-            _ => Err(AfError::from(err_val)),
-        }
+        HANDLE_ERROR(AfError::from(err_val));
+        Array::from(temp)
     }
 }
 
 /// Set seed for random number generation
-pub fn set_seed(seed: u64) -> Result<(), AfError> {
+pub fn set_seed(seed: u64) {
     unsafe {
         let err_val = af_set_seed(seed as Uintl);
-        match err_val {
-            0 => Ok(()),
-            _ => Err(AfError::from(err_val)),
-        }
+        HANDLE_ERROR(AfError::from(err_val));
     }
 }
 
 /// Get the seed of random number generator
 #[allow(unused_mut)]
-pub fn get_seed() -> Result<u64, AfError> {
+pub fn get_seed() -> u64 {
     unsafe {
         let mut temp: u64 = 0;
         let err_val = af_get_seed(&mut temp as *mut Uintl);
-        match err_val {
-            0 => Ok(temp),
-            _ => Err(AfError::from(err_val)),
-        }
+        HANDLE_ERROR(AfError::from(err_val));
+        temp
     }
 }
 
 macro_rules! data_gen_def {
     ($fn_name:ident, $ffi_name: ident) => (
         #[allow(unused_mut)]
-        pub fn $fn_name(dims: Dim4, aftype: Aftype) -> Result<Array, AfError> {
+        pub fn $fn_name<T: HasAfEnum>(dims: Dim4) -> Array {
             unsafe {
+                let aftype = T::get_af_dtype();
                 let mut temp: i64 = 0;
                 let err_val = $ffi_name(&mut temp as MutAfArray,
                                         dims.ndims() as c_uint, dims.get().as_ptr() as *const DimT,
                                         aftype as uint8_t);
-                match err_val {
-                    0 => Ok(Array::from(temp)),
-                    _ => Err(AfError::from(err_val)),
-                }
+                HANDLE_ERROR(AfError::from(err_val));
+                Array::from(temp)
             }
         }
     )
@@ -324,14 +302,12 @@ data_gen_def!(identity, af_identity);
 ///
 /// An Array with values as a diagonal Matrix
 #[allow(unused_mut)]
-pub fn diag_create(input: &Array, dim: i32) -> Result<Array, AfError> {
+pub fn diag_create(input: &Array, dim: i32) -> Array {
     unsafe {
         let mut temp: i64 = 0;
         let err_val = af_diag_create(&mut temp as MutAfArray, input.get() as AfArray, dim as c_int);
-        match err_val {
-            0 => Ok(Array::from(temp)),
-            _ => Err(AfError::from(err_val)),
-        }
+        HANDLE_ERROR(AfError::from(err_val));
+        Array::from(temp)
     }
 }
 
@@ -346,15 +322,13 @@ pub fn diag_create(input: &Array, dim: i32) -> Result<Array, AfError> {
 ///
 /// An Array with values of the diagonal from input Array
 #[allow(unused_mut)]
-pub fn diag_extract(input: &Array, dim: i32) -> Result<Array, AfError> {
+pub fn diag_extract(input: &Array, dim: i32) -> Array {
     unsafe {
         let mut temp: i64 = 0;
         let err_val = af_diag_extract(&mut temp as MutAfArray,
                                       input.get() as AfArray, dim as c_int);
-        match err_val {
-            0 => Ok(Array::from(temp)),
-            _ => Err(AfError::from(err_val)),
-        }
+        HANDLE_ERROR(AfError::from(err_val));
+        Array::from(temp)
     }
 }
 
@@ -370,15 +344,13 @@ pub fn diag_extract(input: &Array, dim: i32) -> Result<Array, AfError> {
 ///
 /// Concatenated Array
 #[allow(unused_mut)]
-pub fn join(dim: i32, first: &Array, second: &Array) -> Result<Array, AfError> {
+pub fn join(dim: i32, first: &Array, second: &Array) -> Array {
     unsafe {
         let mut temp: i64 = 0;
         let err_val = af_join(&mut temp as MutAfArray, dim as c_int,
                               first.get() as AfArray, second.get() as AfArray);
-        match err_val {
-            0 => Ok(Array::from(temp)),
-            _ => Err(AfError::from(err_val)),
-        }
+        HANDLE_ERROR(AfError::from(err_val));
+        Array::from(temp)
     }
 }
 
@@ -393,70 +365,31 @@ pub fn join(dim: i32, first: &Array, second: &Array) -> Result<Array, AfError> {
 ///
 /// Concatenated Array
 #[allow(unused_mut)]
-pub fn join_many(dim: i32, inputs: Vec<&Array>) -> Result<Array, AfError> {
+pub fn join_many(dim: i32, inputs: Vec<&Array>) -> Array {
     unsafe {
+        let mut v = Vec::new();
+        for i in inputs {
+            v.push(i.get());
+        }
         let mut temp: i64 = 0;
         let err_val = af_join_many(&mut temp as MutAfArray, dim as c_int,
-                                   inputs.len() as c_uint, inputs.as_ptr() as *const AfArray);
-        match err_val {
-            0 => Ok(Array::from(temp)),
-            _ => Err(AfError::from(err_val)),
-        }
+                                   v.len() as c_uint, v.as_ptr() as *const AfArray);
+        HANDLE_ERROR(AfError::from(err_val));
+        Array::from(temp)
     }
 }
 
-/// Join multiple Arrays along a given dimension
-///
-/// # Examples
-///
-/// ```
-/// # #[macro_use] extern crate arrayfire;
-/// # fn main() {
-/// let a = randu(Dim4::new(&[5, 3, 1, 1]), Aftype::F32).unwrap();
-/// let b = randu(Dim4::new(&[5, 3, 1, 1]), Aftype::F32).unwrap();
-/// let c = randu(Dim4::new(&[5, 3, 1, 1]), Aftype::F32).unwrap();
-/// let d = join_many![2, a, b, c];
-/// # }
-/// ```
-///
-/// # Panics
-///
-/// Using this macro to create an Array from multiple Arrays doesn't implicitly check for errors
-/// during FFI calls. Therefore, make sure your inputs are correct. In case, you do need proper
-/// error checking, use the [function version](./fn.join_many.html) of this macro.
-// Using macro to implement join many wrapper
-#[macro_export]
-macro_rules! join_many {
-    [$dim: expr; $($x:ident),+] => {
-        {
-            let mut temp_vec = Vec::new();
-            $(
-                temp_vec.push($x);
-             )*
-            let mut temp: i64 = 0;
-            unsafe {
-                let mut temp: i64 = 0;
-                af_join_many(&mut temp as MutAfArray, $dim as c_int,
-                             temp_vec.len() as c_uint, temp_vec.as_ptr() as *const AfArray);
-                Array::from(temp)
-            }
-        }
-    };
-}
-
 macro_rules! data_func_def {
     ($fn_name:ident, $ffi_name: ident) => (
         #[allow(unused_mut)]
-        pub fn $fn_name(input: &Array, dims: Dim4) -> Result<Array, AfError> {
+        pub fn $fn_name(input: &Array, dims: Dim4) -> Array {
             unsafe {
                 let mut temp: i64 = 0;
                 let err_val = $ffi_name(&mut temp as MutAfArray, input.get() as AfArray,
                                         dims[0] as c_uint, dims[1] as c_uint,
                                         dims[2] as c_uint, dims[3] as c_uint);
-                match err_val {
-                    0 => Ok(Array::from(temp)),
-                    _ => Err(AfError::from(err_val)),
-                }
+                HANDLE_ERROR(AfError::from(err_val));
+                Array::from(temp)
             }
         }
     )
@@ -476,28 +409,24 @@ data_func_def!(shift, af_shift);
 /// # Return Values
 /// Reshaped Array
 #[allow(unused_mut)]
-pub fn moddims(input: &Array, dims: Dim4) -> Result<Array, AfError> {
+pub fn moddims(input: &Array, dims: Dim4) -> Array {
     unsafe {
         let mut temp: i64 = 0;
         let err_val = af_moddims(&mut temp as MutAfArray, input.get() as AfArray,
                                  dims.ndims() as c_uint, dims.get().as_ptr() as *const DimT);
-        match err_val {
-            0 => Ok(Array::from(temp)),
-            _ => Err(AfError::from(err_val)),
-        }
+        HANDLE_ERROR(AfError::from(err_val));
+        Array::from(temp)
     }
 }
 
 /// Flatten the multidimensional Array to an 1D Array
 #[allow(unused_mut)]
-pub fn flat(input: &Array) -> Result<Array, AfError> {
+pub fn flat(input: &Array) -> Array {
     unsafe {
         let mut temp: i64 = 0;
         let err_val = af_flat(&mut temp as MutAfArray, input.get() as AfArray);
-        match err_val {
-            0 => Ok(Array::from(temp)),
-            _ => Err(AfError::from(err_val)),
-        }
+        HANDLE_ERROR(AfError::from(err_val));
+        Array::from(temp)
     }
 }
 
@@ -512,14 +441,12 @@ pub fn flat(input: &Array) -> Result<Array, AfError> {
 ///
 /// Flipped Array
 #[allow(unused_mut)]
-pub fn flip(input: &Array, dim: u32) -> Result<Array, AfError> {
+pub fn flip(input: &Array, dim: u32) -> Array {
     unsafe {
         let mut temp: i64 = 0;
         let err_val = af_flip(&mut temp as MutAfArray, input.get() as AfArray, dim as c_uint);
-        match err_val {
-            0 => Ok(Array::from(temp)),
-            _ => Err(AfError::from(err_val)),
-        }
+        HANDLE_ERROR(AfError::from(err_val));
+        Array::from(temp)
     }
 }
 
@@ -533,15 +460,13 @@ pub fn flip(input: &Array, dim: u32) -> Result<Array, AfError> {
 /// # Return Values
 /// Array
 #[allow(unused_mut)]
-pub fn lower(input: &Array, is_unit_diag: bool) -> Result<Array, AfError> {
+pub fn lower(input: &Array, is_unit_diag: bool) -> Array {
     unsafe {
         let mut temp: i64 = 0;
         let err_val = af_lower(&mut temp as MutAfArray,
                                input.get() as AfArray, is_unit_diag as c_int);
-        match err_val {
-            0 => Ok(Array::from(temp)),
-            _ => Err(AfError::from(err_val)),
-        }
+        HANDLE_ERROR(AfError::from(err_val));
+        Array::from(temp)
     }
 }
 
@@ -555,15 +480,13 @@ pub fn lower(input: &Array, is_unit_diag: bool) -> Result<Array, AfError> {
 /// # Return Values
 /// Array
 #[allow(unused_mut)]
-pub fn upper(input: &Array, is_unit_diag: bool) -> Result<Array, AfError> {
+pub fn upper(input: &Array, is_unit_diag: bool) -> Array {
     unsafe {
         let mut temp: i64 = 0;
         let err_val = af_upper(&mut temp as MutAfArray,
                                input.get() as AfArray, is_unit_diag as c_int);
-        match err_val {
-            0 => Ok(Array::from(temp)),
-            _ => Err(AfError::from(err_val)),
-        }
+        HANDLE_ERROR(AfError::from(err_val));
+        Array::from(temp)
     }
 }
 
@@ -572,7 +495,7 @@ pub fn upper(input: &Array, is_unit_diag: bool) -> Result<Array, AfError> {
 /// This function does the C-equivalent of the following statement, except that the operation
 /// happens on a GPU for all elements simultaneously.
 ///
-/// ```
+/// ```ignore
 /// c = cond ? a : b; /// where cond, a & b are all objects of type Array
 /// ```
 ///
@@ -588,15 +511,13 @@ pub fn upper(input: &Array, is_unit_diag: bool) -> Result<Array, AfError> {
 ///
 /// An Array
 #[allow(unused_mut)]
-pub fn select(a: &Array, cond: &Array, b: &Array) -> Result<Array, AfError> {
+pub fn select(a: &Array, cond: &Array, b: &Array) -> Array {
     unsafe {
         let mut temp: i64 = 0;
         let err_val = af_select(&mut temp as MutAfArray, cond.get() as AfArray,
-        a.get() as AfArray, b.get() as AfArray);
-        match err_val {
-            0 => Ok(Array::from(temp)),
-            _ => Err(AfError::from(err_val)),
-        }
+                                a.get() as AfArray, b.get() as AfArray);
+        HANDLE_ERROR(AfError::from(err_val));
+        Array::from(temp)
     }
 }
 
@@ -605,7 +526,7 @@ pub fn select(a: &Array, cond: &Array, b: &Array) -> Result<Array, AfError> {
 /// This function does the C-equivalent of the following statement, except that the operation
 /// happens on a GPU for all elements simultaneously.
 ///
-/// ```
+/// ```ignore
 /// c = cond ? a : b; /// where  a is a scalar(f64) and b is Array
 /// ```
 ///
@@ -621,15 +542,13 @@ pub fn select(a: &Array, cond: &Array, b: &Array) -> Result<Array, AfError> {
 ///
 /// An Array
 #[allow(unused_mut)]
-pub fn selectl(a: f64, cond: &Array, b: &Array) -> Result<Array, AfError> {
+pub fn selectl(a: f64, cond: &Array, b: &Array) -> Array {
     unsafe {
         let mut temp: i64 = 0;
         let err_val = af_select_scalar_l(&mut temp as MutAfArray, cond.get() as AfArray,
         a as c_double, b.get() as AfArray);
-        match err_val {
-            0 => Ok(Array::from(temp)),
-            _ => Err(AfError::from(err_val)),
-        }
+        HANDLE_ERROR(AfError::from(err_val));
+        Array::from(temp)
     }
 }
 
@@ -638,7 +557,7 @@ pub fn selectl(a: f64, cond: &Array, b: &Array) -> Result<Array, AfError> {
 /// This function does the C-equivalent of the following statement, except that the operation
 /// happens on a GPU for all elements simultaneously.
 ///
-/// ```
+/// ```ignore
 /// c = cond ? a : b; /// where a is Array and b is a scalar(f64)
 /// ```
 ///
@@ -654,15 +573,13 @@ pub fn selectl(a: f64, cond: &Array, b: &Array) -> Result<Array, AfError> {
 ///
 /// An Array
 #[allow(unused_mut)]
-pub fn selectr(a: &Array, cond: &Array, b: f64) -> Result<Array, AfError> {
+pub fn selectr(a: &Array, cond: &Array, b: f64) -> Array {
     unsafe {
         let mut temp: i64 = 0;
         let err_val = af_select_scalar_r(&mut temp as MutAfArray, cond.get() as AfArray,
-        a.get() as AfArray, b as c_double);
-        match err_val {
-            0 => Ok(Array::from(temp)),
-            _ => Err(AfError::from(err_val)),
-        }
+                                         a.get() as AfArray, b as c_double);
+        HANDLE_ERROR(AfError::from(err_val));
+        Array::from(temp)
     }
 }
 
@@ -671,7 +588,7 @@ pub fn selectr(a: &Array, cond: &Array, b: f64) -> Result<Array, AfError> {
 /// This function does the C-equivalent of the following statement, except that the operation
 /// happens on a GPU for all elements simultaneously.
 ///
-/// ```
+/// ```ignore
 /// a = cond ? a : b; /// where cond, a & b are all objects of type Array
 /// ```
 ///
@@ -684,15 +601,12 @@ pub fn selectr(a: &Array, cond: &Array, b: f64) -> Result<Array, AfError> {
 ///
 /// # Return Values
 ///
-/// An Array
+/// None
 #[allow(unused_mut)]
-pub fn replace(a: &mut Array, cond: &Array, b: &Array) -> Result<(), AfError> {
+pub fn replace(a: &mut Array, cond: &Array, b: &Array) {
     unsafe {
         let err_val = af_replace(a.get() as AfArray, cond.get() as AfArray, b.get() as AfArray);
-        match err_val {
-            0 => Ok(()),
-            _ => Err(AfError::from(err_val)),
-        }
+        HANDLE_ERROR(AfError::from(err_val));
     }
 }
 
@@ -701,7 +615,7 @@ pub fn replace(a: &mut Array, cond: &Array, b: &Array) -> Result<(), AfError> {
 /// This function does the C-equivalent of the following statement, except that the operation
 /// happens on a GPU for all elements simultaneously.
 ///
-/// ```
+/// ```ignore
 /// a = cond ? a : b; /// where cond, a are Arrays and b is scalar(f64)
 /// ```
 ///
@@ -714,14 +628,11 @@ pub fn replace(a: &mut Array, cond: &Array, b: &Array) -> Result<(), AfError> {
 ///
 /// # Return Values
 ///
-/// An Array
+/// None
 #[allow(unused_mut)]
-pub fn replace_scalar(a: &mut Array, cond: &Array, b: f64) -> Result<(), AfError> {
+pub fn replace_scalar(a: &mut Array, cond: &Array, b: f64) {
     unsafe {
         let err_val = af_replace_scalar(a.get() as AfArray, cond.get() as AfArray, b as c_double);
-        match err_val {
-            0 => Ok(()),
-            _ => Err(AfError::from(err_val)),
-        }
+        HANDLE_ERROR(AfError::from(err_val));
     }
-}
+}
\ No newline at end of file
